@@ -31,7 +31,6 @@ The parser offers two methods:
 
 import logging
 import re
-from collections.abc import Iterator
 from dataclasses import dataclass
 from pathlib import Path
 
@@ -43,6 +42,36 @@ LOGGER = logging.getLogger(__name__)
 RE_CON = re.compile(r"\s*\.(?P<port>.*)\s*\((?P<con>.*)\)\s*(//(?P<comment>.*))?")
 
 
+@dataclass
+class _ConDeclaration:
+    """Connection.
+
+    Attributes:
+        port: Port
+        con: Connection
+        comment: Comment
+    """
+
+    port: str = ""
+    con: str = ""
+    comment: list[str] | None = None
+    ifdefs: list[str] | None = None
+
+    def proc_tokens(self, token, string):
+        if token == ("Port",):
+            self.port = string
+        elif token == ("Connection",):
+            self.con = string
+        elif token == ("PortConnection",):
+            self.port = string
+            self.con = string
+        elif token == ("Comment",):
+            if self.comment is None:
+                self.comment = [string]
+            else:
+                self.comment.append(string)
+
+
 @dataclass
 class _ModInstance:
     """Represents an instance of a module within another module.
@@ -55,13 +84,24 @@ class _ModInstance:
 
     name: str | None = None
     module: str | None = None
-    connections: tuple[dm.Connection, ...] | None = None
+    connections: list[_ConDeclaration] | None = None
 
-    def proc_tokens(self, token, string):
+    def proc_tokens(self, token, string, ifdefs):
         if token == Module.Body.Instance.Name:
             self.name = string
-        elif token == Module.Body.Instance.Connections:
-            self.connections = tuple(_normalize_connections(string))
+        elif token == Module.Body.Instance.Con.Start:
+            if self.connections is None:
+                self.connections = [_ConDeclaration(ifdefs=ifdefs)]
+            else:
+                self.connections.append(_ConDeclaration(ifdefs=ifdefs))
+        elif token == Module.Body.Instance.Con.OrderedConnection:
+            if self.connections is None:
+                self.connections = [_ConDeclaration(con=string, ifdefs=ifdefs)]
+            else:
+                self.connections.append(_ConDeclaration(con=string, ifdefs=ifdefs))
+        elif token[:4] == Module.Body.Instance.Con:
+            if self.connections is not None:  # Con.Comment can trigger this
+                self.connections[-1].proc_tokens(token[4:], string)
 
 
 @dataclass
@@ -78,20 +118,25 @@ class _PortDeclaration:
 
     direction: str
     ptype: str | None = None
+    dtype: str | None = None
     name: list[str] | None = None
     dim: str | None = None
     dim_unpacked: str | None = None
     comment: list[str] | None = None
+    ifdefs: list[str] | None = None
 
-    def proc_tokens(self, token, string):
+    def proc_tokens(self, token, string, ifdefs):  # noqa: C901
         """Processes Module.Port tokens and extract data."""
         if token == Module.Port.PortDirection:
             self.direction = string
-        elif token == Module.Port.PortType:
+        elif token == Module.Port.Ptype:
             self.ptype = string
+        elif token == Module.Port.Dtype:
+            self.dtype = string
         elif token == Module.Port.PortName:
             if self.name is None:
                 self.name = [string]
+                self.ifdefs = ifdefs
             else:
                 self.name.append(string)
         elif token == Module.Port.PortWidth:
@@ -123,19 +168,22 @@ class _ParamDeclaration:
     dim: str | None = None
     dim_unpacked: str | None = None
     comment: list[str] | None = None
+    ifdefs: list[str] | None = None
 
-    def proc_tokens(self, token, string):
+    def proc_tokens(self, token, string, ifdefs):
         """Processes Module.Param tokens and extract data."""
         if token == Module.Param.ParamType:
             self.ptype = string
         elif token == Module.Param.ParamName:
             if self.name is None:
                 self.name = [string]
+                self.ifdefs = ifdefs
             else:
                 self.name.append(string)
         elif token == Module.Param.ParamWidth:
             if self.name is None:
                 self.dim = string
+                self.ifdefs = ifdefs
             elif self.dim_unpacked is None:
                 self.dim_unpacked = string
         elif token == Module.Param.Comment:
@@ -146,22 +194,13 @@ class _ParamDeclaration:
 
 
 def _normalize_comments(comment: list[str]) -> tuple[str, ...]:
-    return tuple(line.removeprefix("//").strip() for line in comment or ())
+    return tuple(line.replace("\n", " ").strip() for line in comment or ())
 
 
-def _normalize_connections(lines: str) -> Iterator[dm.Connection]:
-    # TODO: please use lexer for this hack!
-    for line in lines.splitlines():
-        line = line.strip()  # noqa: PLW2901
-        if not line:
-            continue
-        mat = RE_CON.match(line)
-        if not mat:
-            LOGGER.warning(f"Invalid connection: {line}")
-            continue
-        data = mat.groupdict()
-        data["comment"] = (data["comment"],) if data["comment"] else ()
-        yield dm.Connection(**data)  # type: ignore[arg-type]
+def _flip_ifdef(param):
+    if param.startswith("!"):
+        return param[1:]  # Removes '!' prefix
+    return f"!{param}"  # Adds '!' prefix
 
 
 class _SvModule:
@@ -171,7 +210,7 @@ class _SvModule:
         name: Name of the module
         port_lst: List of Port objects
         param_lst: List of Param objects
-        inst_dict: Dictionary of module instances (instance name -> module)
+        inst_list: List of ModuleInstance objects
         port_decl: List of PortDeclaration objects
         param_decl: List of ParamDeclaration objects
         inst_decl: List of instance declarations
@@ -181,26 +220,27 @@ class _SvModule:
         self.name: str | None = None
         self.port_lst: list[dm.Port] = []
         self.param_lst: list[dm.Param] = []
+        self.inst_lst: list[dm.ModuleInstance] = []
         self.inst_dict: dict[str, str] = {}
 
         self.port_decl: list[_PortDeclaration] = []
         self.param_decl: list[_ParamDeclaration] = []
         self.inst_decl: list[_ModInstance] = []
+        self.ifdefs_stack: list = []
+        self.ifdefs_pop_stack: list = []
 
     def _gen_port_lst(self):
         for decl in self.port_decl:
             for name in decl.name:
-                # TODO: maybe split in parser and not in post-processing
-                ptype = decl.ptype if decl.ptype in ("reg", "wire", "logic") else ""
-                dtype = decl.ptype if decl.ptype in ("signed", "unsigned") else ""
                 port = dm.Port(
                     name=name,
                     direction=decl.direction,
-                    ptype=ptype,
-                    dtype=dtype,
+                    ptype=decl.ptype or "",
+                    dtype=decl.dtype or "",
                     dim=decl.dim or "",
                     dim_unpacked=decl.dim_unpacked or "",
                     comment=_normalize_comments(decl.comment),
+                    ifdefs=tuple(decl.ifdefs),
                 )
                 self.port_lst.append(port)
 
@@ -213,12 +253,51 @@ class _SvModule:
                     dim=decl.dim or "",
                     dim_unpacked=decl.dim_unpacked or "",
                     comment=_normalize_comments(decl.comment),
+                    ifdefs=tuple(decl.ifdefs),
                 )
                 self.param_lst.append(param)
 
-    def _gen_inst_dict(self):
+    def _gen_inst_lst(self):
         for decl in self.inst_decl:
-            self.inst_dict[decl.name] = decl.module
+            inst = dm.ModuleInstance(
+                name=decl.name,
+                module=decl.module,
+                connections=tuple(
+                    dm.Connection(
+                        port=con.port or "",
+                        con=con.con or "",
+                        comment=_normalize_comments(con.comment),
+                        ifdefs=tuple(con.ifdefs),
+                    )
+                    for con in decl.connections
+                ),
+            )
+            self.inst_lst.append(inst)
+
+    def proc_ifdef(self, token, string):
+        # Process the ifdef stack list
+        if token[-1] == "IFDEF":
+            self.ifdefs_stack.append(string)
+        elif token[-1] == "IFNDEF":
+            self.ifdefs_stack.append(_flip_ifdef(string))
+        elif token[-1] == "ELSIF":
+            self.ifdefs_stack[-1] = _flip_ifdef(self.ifdefs_stack[-1])
+            self.ifdefs_stack.append(string)
+        elif token[-1] == "ELSE":
+            self.ifdefs_stack[-1] = _flip_ifdef(self.ifdefs_stack[-1])
+        elif token[-1] == "ENDIF":
+            del self.ifdefs_stack[-self.ifdefs_pop_stack[-1] :]
+
+        # Process the pop stack list
+        if token[-1] in ["IFDEF", "IFNDEF"]:
+            self.ifdefs_pop_stack.append(1)
+        elif token[-1] in ["ELSIF"]:
+            self.ifdefs_pop_stack[-1] += 1
+        elif token[-1] in ["ENDIF"]:
+            del self.ifdefs_pop_stack[-1]
+
+        LOGGER.debug(f"IFDEF stack: {self.ifdefs_stack}")
+        LOGGER.debug(f"IFDEF stack: {self.ifdefs_pop_stack}")
 
     def proc_tokens(self, token, string):
         # Capture a new port declaration object if input/output keywords are found
@@ -226,14 +305,14 @@ class _SvModule:
             if token[-1] == ("PortDirection"):
                 self.port_decl.append(_PortDeclaration(direction=string))
             else:
-                self.port_decl[-1].proc_tokens(token, string)
+                self.port_decl[-1].proc_tokens(token, string, self.ifdefs_stack.copy())
 
         # Capture parameters, when Module.Param tokens are found
         elif token[:2] == ("Module", "Param"):
             if token is Module.Param:
                 self.param_decl.append(_ParamDeclaration())
             else:
-                self.param_decl[-1].proc_tokens(token, string)
+                self.param_decl[-1].proc_tokens(token, string, self.ifdefs_stack.copy())
 
         # Capture Modules
         elif token[:2] == ("Module", "ModuleName"):
@@ -244,7 +323,10 @@ class _SvModule:
             if token == Module.Body.Instance.Module:
                 self.inst_decl.append(_ModInstance(module=string))
             else:
-                self.inst_decl[-1].proc_tokens(token, string)
+                self.inst_decl[-1].proc_tokens(token, string, self.ifdefs_stack.copy())
+
+        elif token[:2] == ("Module", "IFDEF"):
+            self.proc_ifdef(token, string)
 
 
 def parse_file(file_path: Path | str) -> dm.File:
@@ -284,10 +366,7 @@ def parse_text(text: str, file_path: Path | str | None = None) -> dm.File:
             name=mod.name,
             params=mod.param_lst,
             ports=mod.port_lst,
-            insts=tuple(
-                dm.ModuleInstance(name=inst.name, module=inst.module, connections=inst.connections)
-                for inst in mod.inst_decl
-            ),
+            insts=mod.inst_lst,
         )
         for mod in module_lst
     )
@@ -312,6 +391,6 @@ def _parse_text(text: str):
     for mod in module_lst:
         mod._gen_port_lst()
         mod._gen_param_lst()
-        mod._gen_inst_dict()
+        mod._gen_inst_lst()
 
     return module_lst
