@@ -24,10 +24,26 @@
 
 from pathlib import Path
 
+from click.testing import CliRunner
 from pytest import fixture
 
+EXAMPLES_PATH = Path(__file__).parent.parent / "examples"
+EXAMPLES = tuple(sorted(EXAMPLES_PATH.glob("*.sv")))
+
 
 @fixture
 def project_root() -> Path:
     """Path to Project Root."""
     return Path(__file__).parent.parent
+
+
+@fixture
+def examples() -> Path:
+    """Path to Examples."""
+    return EXAMPLES_PATH
+
+
+@fixture
+def runner() -> CliRunner:
+    """CLI Runner."""
+    return CliRunner()
